
// *************************************
// **         NOTES
// *************************************





#include "SerialPort.h"
#include "message.h"
#include <iostream>
//#include "date.h"
#include <fstream>
#include "AnaloggerConfig.h"


message teensyMessage;
SerialPort teensyPort;



int ADCSTART = 1111;

int ADCEND = 9999;

int SWTRIGGER = 5555;

int numberOfTriggers;



// *************************************
// **         MAIN
// *************************************

int main(){
    

    while(!(teensyPort.isConnected())){
            
        teensyPort.connectToPort(portName);

        std::cout << portName << " Connected\n";

    }
    
    teensyPort.flushInput();

    bool needResponse = true;
    
    std::cout << "Waiting for Configuration Message\n";
    
    while (needResponse){
        
        if (teensyMessage.waitForMessage(teensyMessage.Reply, teensyPort, 3)){
            
            teensyMessage.parseMessage(teensyMessage.Reply);
            
            if (teensyMessage.rType == "Echo"){
                
                if (teensyMessage.rValue == 12345){
                    
                    std::cout << "Configuration Message Received\n";
                    
                    teensyMessage.sendMessage(teensyMessage.Reply, teensyPort, "Echo", "Frequency", 54321);
                    
                    std::cout << "Configuration Message Sent\n";
                    
                    needResponse = false;
                    
                }
                
            }
            
        }
        
    }

    std::this_thread::sleep_for(std::chrono::milliseconds(1000));
    
    std::string timeStamp = std::to_string(std::chrono::system_clock::now().time_since_epoch().count());
    //std::string timeStamp = date::format("%F %T", std::chrono::system_clock::now());
    
    std::string fileName = "Log File " + timeStamp;

    std::ofstream log;
    
    log.open (fileName);
    
    std::cout << "Log Opened\n" << "Gathering Configuration Parameters\n";
        
    for(auto const& pairs : teensyMessage.Header){
        
        std::cout << "...\n";
        
        teensyMessage.Header[pairs.first] = teensyMessage.sendMessage(teensyMessage.Reply, teensyPort, "Request", pairs.first, 0);
        
        log << pairs.first + ": " + std::to_string(pairs.second) + "\n";
        
    }
    
    numberOfTriggers = teensyMessage.Header["numTriggers"];
    
    std::cout << "Request Data Mode\n";
    
    while (true){
        
        if(teensyMessage.sendMessage(teensyMessage.Reply, teensyPort, "Push", "ADCStatus", ADCSTART)){
         
            break;
         
        }
        
    }
    
    int lastSer = 0;
    
    while(numberOfTriggers > 0){
    
    if(teensyMessage.Header["SWEnabled"] > 0){
    
        int randomFlag = 1;
        
        std::cout << "Enter anything to Trigger\n";
        
        while (randomFlag){
            
            std::string DUD;
            
            std::cin >> DUD;
            
            if(teensyMessage.sendMessage(teensyMessage.Reply, teensyPort, "Push", "ADCStatus", SWTRIGGER)){
            
                randomFlag = 0;
                
            }
        }
    }  
    
        
    
        std::cout << "Collecting Data...\n";

        bool runFlag = true;

        while(runFlag){
  
            bool check = teensyMessage.waitForMessage(teensyMessage.Reply, teensyPort, 1);
            
            if(check){
                
                teensyMessage.parseMessage(teensyMessage.Reply);
                
                if(teensyMessage.rType == "Data"){

                    if(teensyMessage.SerialNum - lastSer == 1){

                        log << std::to_string(teensyMessage.SerialNum) + ": ";

                        if(teensyMessage.Header["Frequency"] > 450000){

                            for( int ii = 0; ii < 60; ii++){

                                log << std::to_string(teensyMessage.MessData[ii]) + " ";
                                
                            }
                        }
                            
                        else{

                            // Parse 12 Bit
                            for(int ii = 0; ii < 60; ii += 3){

                                int writeNum = teensyMessage.MessData[ii] + ((teensyMessage.MessData[ii + 1] & 0xF0) << 4);
                                
                                int writeNum2 = ((teensyMessage.MessData[ii+1] & 0x0F) << 8) + teensyMessage.MessData[ii + 2];
                                
                                log << std::to_string(writeNum) + " " + std::to_string(writeNum2) + " ";
                                
                            }
                        }
                    
                        log << "\n";
                        
                    }

                    else{

                        for( int jj = lastSer + 1; jj < teensyMessage.SerialNum; jj++){

                            if(teensyMessage.Header["Frequency"] > 450000){

                                log << std::to_string(jj) + ": -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1" + "\n";
                                
                            }

                            else{

                                log << std::to_string(jj) + ": -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1" + "\n";
                                
                            }
                            
                        }
                            
                        log << std::to_string(teensyMessage.SerialNum) + ": ";

                        if(teensyMessage.Header["Frequency"] > 450000){

                            for( int ii = 0; ii < 60; ii++){

                                log << std::to_string(teensyMessage.MessData[ii]) + " ";
                                
                            }
                        }
                            
                        else{

                            // Parse 12 Bit
                            for(int ii = 0; ii < 60; ii += 3){

                                int writeNum = teensyMessage.MessData[ii] + ((teensyMessage.MessData[ii + 1] & 0xF0) << 4);
                                
                                int writeNum2 = ((teensyMessage.MessData[ii+1] & 0x0F) << 8) + teensyMessage.MessData[ii + 2];
                                
                                log << std::to_string(writeNum) + " " + std::to_string(writeNum2) + " ";
                                
                            }
                        }
                    
                        log << "\n";
                        //}
                        
                    }

                    lastSer = teensyMessage.SerialNum;
                    
                }
            
                else if ((teensyMessage.rType == "Push") && (teensyMessage.rSubType == "ADCStatus") && (teensyMessage.rValue == ADCEND)){
                    
                    //std::string endtimeStamp = date::format("%F %T", std::chrono::system_clock::now());
                    std::string endtimeStamp = std::to_string(std::chrono::system_clock::now().time_since_epoch().count());
                    
                    log << "Stopped at " + endtimeStamp + "\n";
                    
                    runFlag = false;
                    
                    std::cout << "Data Collection Finished\n";
                    
                    bool error_message = teensyMessage.waitForMessage(teensyMessage.Reply, teensyPort, 1);
                    
                    if (error_message){
                        
                        teensyMessage.parseMessage(teensyMessage.Reply);
                        
                        if ((teensyMessage.rType == "Push") && (teensyMessage.rSubType == "ADCStatus")){
                        
                                log << "Last overrun occured after reading: " << teensyMessage.rValue << "\n";
                            
                        }
                        
                    }
                    
                }
                
            }
        
        }
        
        numberOfTriggers --;
        
    }
        
            
    teensyPort.closePort();   
    
    std::cout << "Port Closed\n";
    
    log.close();
    
    std::cout << "Log Closed\n";

    std::this_thread::sleep_for(std::chrono::milliseconds(1000));
    
    return 0;

    //Add plot log function at some point
}


