#include "SerialPort.h"
#include <stdio.h>
#ifndef iostream
#include <iostream>
#endif

#ifdef _WIN32

SerialPort::SerialPort(){
    
     this->connected = false;
    
}

void SerialPort::connectToPort(char *portName){

    this->handler = CreateFileA(static_cast<LPCSTR>(portName),
                                GENERIC_READ | GENERIC_WRITE,
                                0,
                                NULL,
                                OPEN_EXISTING,
                                FILE_ATTRIBUTE_NORMAL,
                                NULL);
    if (this->handler == INVALID_HANDLE_VALUE){
        if (GetLastError() == ERROR_FILE_NOT_FOUND){
            printf("ERROR: Handle was not attached. Reason: %s not available\n", portName);
        }
    else
        {
            printf("ERROR!!!");
        }
    }
    else {
        DCB dcbSerialParameters = {0};

        if (!GetCommState(this->handler, &dcbSerialParameters)) {
            printf("failed to get current serial parameters");
        }
        else {
            dcbSerialParameters.BaudRate = 460800;
            dcbSerialParameters.ByteSize = 8;
            dcbSerialParameters.StopBits = ONESTOPBIT;
            dcbSerialParameters.Parity = NOPARITY;
            dcbSerialParameters.fDtrControl = DTR_CONTROL_ENABLE;

            if (!SetCommState(handler, &dcbSerialParameters))
            {
                printf("ALERT: could not set Serial port parameters\n");
            }
            else {
                this->connected = true;
                PurgeComm(this->handler, PURGE_RXCLEAR | PURGE_TXCLEAR);
                Sleep(ARDUINO_WAIT_TIME);
            }
        }
    }
}

void SerialPort::closePort(){
    
    if (this->connected){
        
        this->connected = false;
        
        CloseHandle(this->handler);
        
    }
    
}


int SerialPort::readSerialPort(unsigned char *buffer, unsigned int buf_size){
    
    DWORD bytesRead;
    
    unsigned int toRead;

    ClearCommError(this->handler, &this->errors, &this->status);

    if (this->status.cbInQue > 0){
        
        if (this->status.cbInQue > buf_size){
            
            toRead = buf_size;
            
        }
        
        else toRead = this->status.cbInQue;
    }

    if (ReadFile(this->handler, buffer, toRead, &bytesRead, NULL)) return bytesRead;

    return 0;
}


bool SerialPort::writeSerialPort(unsigned char *buffer, unsigned int buf_size){
    
    DWORD bytesSend;

    if (!WriteFile(this->handler, (void*) buffer, buf_size, &bytesSend, 0)){
        
        ClearCommError(this->handler, &this->errors, &this->status);
        
        return false;
        
    }
    
    else return true;
    
}

bool SerialPort::isConnected(){
    
    return this->connected;
    
}

void SerialPort::flushInput(){
    
    PurgeComm(this->handler, PURGE_RXCLEAR);

}



#endif

#ifdef __linux__

SerialPort::SerialPort(){
    
    this->connected = false;
    
}

void SerialPort::connectToPort(char *portName) {
    
    portFile = open (portName, O_RDWR | O_NOCTTY | O_SYNC);
    
    if (portFile < 0) {
        
        printf("Error Opening Port: %s\n", strerror(errno));
        
        return;
        
    }
    
    setAttributes (portFile, B460800);
    
    this->connected = true;
    
}

void SerialPort::closePort() {
    
    if (SerialPort::connected){
        
        SerialPort::connected = false;
        
        close(SerialPort::portFile);
        
    }
}

int SerialPort::setAttributes (int portFile, int speed) {
    
        struct termios tty;
        
        memset (&tty, 0, sizeof tty);
        
        if (tcgetattr (portFile, &tty) != 0)
        {
            
                printf("Error from tcgetattr: %s\n", strerror(errno));
                
                return -1;
        }

        cfsetospeed (&tty, speed);
        
        cfsetispeed (&tty, speed);

        tty.c_cflag = (tty.c_cflag & ~CSIZE) | CS8;     // 8-bit characters
        
        tty.c_cflag |= (CLOCAL | CREAD);

        tty.c_cflag &= ~PARENB;
        
        tty.c_cflag &= ~CSTOPB;
        
        tty.c_cflag &= ~CRTSCTS; //Disable HW Flow control if exists
        
        tty.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);

        tty.c_iflag &= ~(IXON | IXOFF | IXANY); // shut off xon/xoff ctrl
        
        tty.c_iflag &= ~(IGNBRK | BRKINT | PARMRK| INLCR | IGNCR | ICRNL); // No characters are special. NONE!
        
        tty.c_oflag &= ~OPOST;          //Raw Output
        
        tty.c_cc[VMIN]  = 0;            // read doesn't block, lies.  Read blocks.  Always blocks.
        
        tty.c_cc[VTIME] = 10;            // 1 second read timeout, doesn't seem to work in linux anyway, read seems to block regardless


        if (tcsetattr (portFile, TCSANOW, &tty) != 0)
        {
                printf("Error from tcgetattr: %s\n", strerror(errno));
                
                return -1;
                
        }
        return 0;
}

int SerialPort::readSerialPort(unsigned char *buffer, unsigned int buf_size)
{    
        int readAttempt = read (SerialPort::portFile, buffer, buf_size);

        if (readAttempt == buf_size){

            return buf_size;
        
        }    
        else {

            return 0;
        
        }
    
}

bool SerialPort::writeSerialPort(unsigned char *buffer, unsigned int buf_size)
{
   int writeAttempt =  write(SerialPort::portFile, buffer, buf_size);
    

    if (writeAttempt < buf_size){

        return false;
    }
    
    else return true;
    
}

bool SerialPort::isConnected() {
    
    return this->connected;
    
}

void SerialPort::flushInput(){
    
    tcflush(SerialPort::portFile, TCIFLUSH);
    
}

#endif

