// **********************************************
// **        NOTES
// **********************************************

/*
 * Analogger is a simple datalogging script that uses the teensy as a single channel adc.  The goal is to have a simple interface that takes a few configuration parameters and 
 * does the work of setting up the adc and communication with a host computer.  The adc setup itself takes one parameter, frequency.  Based on this,
 * the adc is setup loosely based on the speed tests done by Pedvide in the continuous read sample script that ships with teensyduino.
 * For the datalogging, the other parameters are whether or not a hardware trigger, or a software trigger is used to start the logging, how many times you
 * want the logger to trigger, and how much time is spent recording after each trigger.  Analogger defaults to frequency = 10000, duration = 1 sec, number of triggers = 1, and trigger delay = 0 sec;
 * Analogger configuration is analogger.config(Software trigger flag, frequency, duration, number of triggers, trigger delay). Any number of these can be left as default as long as they are specified appropriately.
 * 
 * Speed tests:
 * FOR TEENSY 3.6  ->  Standard Speed    ::    Roughly 850,000 Samples per second was my record
 *                 ->  MAX Speed (240MHz)::    1,000,000  Samples per second was my record (max allowed by analogger library)
 * 
 * FOR TEENSY 3.2  ->  Standard Speed    ::    Roughly 300,000 Samples per second
 *                 ->  MAX Speed (120MHz)::    Roughly 350,000 @ 12 bits, or 500,000 @ 8 bits
*/



#include <Analogger.h>

Analogger hwLogger = Analogger();


//Pins to write,  These are generally for testing purposes.  Comment out if not using the same teensy for signal generation and receipt.
const uint8_t out_pin = 2;          //Pin to put an analog wave to.  Can measure to determine performance of analogger.
const uint8_t digital_out_pin = 6;  //If you want a digital pin to write to in order to trigger the analogger by hardware.


void setup() {
    
    //For test purposes only. Comment if not using for hardware tests
    pinMode(digital_out_pin, OUTPUT);
    digitalWriteFast(digital_out_pin, HIGH);
    analogWriteFrequency(out_pin, 5000);
    analogWrite(out_pin, 128);
    
//USE> AnaloggerInstance.Config(SW trigger (true/false), Frequency (samples per second), Sample Time (milliseconds), Number of Triggers (integer), Trigger Delay (milliseconds));

    hwLogger.Config(true, 100000, 3000, 3, 0);
    
}

void loop() {
    
    //hwLogger.hardwareTrigger();

    hwLogger.softwareTrigger();
    
    
}



void yield(){}

void adc0_isr() {

    hwLogger.isrFunc();
   
}

void pdb_isr(void) {
    
    PDB0_SC &=~PDB_SC_PDBIF; // clear interrupt
    
}
