#include "Analogger.h"
#include <RingBuffer.h>
#include "Arduino.h"



Analogger::Analogger(){

//Configuration Parameters
config_enabled = 0;     //Does the program enter configuration phase at beginning of loop?
hw_trigger_enable = true;   //Does the program activate logging on a hardware trigger?  Note: currently only accepts digitalHigh as a trigger.  Cannot trigger on low.              $$$$
sw_trigger_enable = false;  //Does the program activate logging on a software trigger?                                                                                              $$$$
AnaloggerStatus = 0;
num_of_triggers = 1;    //Number of times the logger can be triggered.  Note: If both hardware and software triggering are enabled, this may not behave as you expect.
sample_resolution = 8;  //Resoultion of each analog reading.  If using default values, choices are 8 or 12.  The number here don't matter, will be changed during setup regardless.
num_avgs = 1;            //Number of averages of each analog reading.  If slow, this may be as high as 32, if at all fast, 1.  The number here don't matter, will be changed during setup regardless.


isConnected = false;

//Buffer Values
write_pos = 0;
read_pos = 0;
adc_val = 0;
tx_buffer_index = 4;

replyType = 0;
replySub = 0;
replyValue = 0;

//These are flags to tell the teensy when to stop recording
start_read = true;
overrun = false;
notTriggered = true;

//These are for buffer overflow conditions.
readCounter = 0;
error_count = 0;
randomCounter = false;

Serial_Number = 1;

//Frequnecy to sample at
pdb_trigger_frequency = 300000;

//Time to sample for per trigger event, I did this instead of defining samples, seems like it's more useful this way. If you don't like it, swap this for the calculations of numSamples in the setup.
seconds = 5;

//Should there be a time delay after trigger before recording? measured in millis
time_delay = 0;

//time trigger is activated in millis.  Used with time_delay.
time_trigger_start = 0;

  
}


void Analogger::isrFunc() {

    buffer->write(adc.adc0->readSingle());

    //If speed is a concern, and we don't want to check for overruns, comment this rest of this out.  This should not be done 100K's of times per second if not needed.
    //Otherwise, if you are checking your overruns, this can be changed to check various things.  Use the check that makes the most sense for your project.
    if (buffer->isFull()){
      
        //if the buffer fills, it will almost certainly overrun, but nothing is stopping the code from continuing.  We just make note of it and move on.
        overrun = true;

        //If we have already overrun, we don't want to keep counting.
        if (randomCounter == false){ 
          
            error_count ++;
            
           randomCounter = true;
           
        }
        
    }

    //If we have caught up, and are no longer overrunning, reset the flag and wait for the next overrun.
    else{

        randomCounter = false;
        
    }
   
}

void Analogger::ADCRecord(RingBuffer *buffer, uint8_t TXbuffer[64], uint8_t &TXindex, uint8_t resolution){

    adc_val = buffer->read();
                        
    //The following block splits readings into bit numbers in the buffer.
    if (resolution == 8){
            
        TXbuffer[TXindex] = adc_val;
        
        TXindex++;
        
    }
    
    else {
        //This splits each reading into two adjacent buffer values.  Since we are attempting to maximize the amount of data sent over the pipe, each second index (starting from first index = 0) of three indecies will actually be a combination of two readings. That is,  Each pair of readings will cross three buffer values with the middle one sharing 4 bits between two readings. For example;  if the first reading were 45, and the second value 3095, the first three values of the buffer will be:  [0x2D, 0x0C, 0x17...   Since the Teensy can't really measure more than 12 bits anyway, this seemed a good plan.
        if (TXindex%3 == 1){
            
            TXbuffer[TXindex] = (0x00ff&adc_val);
            
            TXindex ++;
            
            TXbuffer[TXindex] = ((0x00f0) & (adc_val>>4));
            
        }
        
        else {
            
            TXbuffer[TXindex] = TXbuffer[TXindex] + (0x000f & adc_val >> 8);
            
            TXindex ++;
            
            TXbuffer[tx_buffer_index] = ((0x00ff) & (adc_val));
            
            TXindex ++;
            
        }
        
    }
    
}

void Analogger::adc_setup(uint32_t sampleFrequency){
    
    if (sampleFrequency > 1000000){
        
        sampleFrequency = 1000000;
        
    }
    
    if (sampleFrequency < 1){
        
        sampleFrequency = 1;
        
    }
    
    
    if (sampleFrequency > 600000){
    //Use Defaults for 600kSPS - 1MSPS
        adc.setAveraging(num_avgs);
        adc.setResolution(sample_resolution);
    
        adc.setConversionSpeed(ADC_CONVERSION_SPEED::VERY_HIGH_SPEED);
        adc.setSamplingSpeed(ADC_SAMPLING_SPEED::VERY_HIGH_SPEED);
        
    }
    else if (sampleFrequency > 450000){
        //450kSPS - 600kSPS
        adc.setAveraging(num_avgs);
        adc.setResolution(sample_resolution);
    
        adc.setConversionSpeed(ADC_CONVERSION_SPEED::HIGH_SPEED);
        adc.setSamplingSpeed(ADC_SAMPLING_SPEED::HIGH_SPEED);
        
    }
    else if (sampleFrequency > 220000){
        //220kSPS - 450kSPS
        
        sample_resolution = 12;
        
        adc.setAveraging(num_avgs);
        adc.setResolution(sample_resolution);
    
        adc.setConversionSpeed(ADC_CONVERSION_SPEED::HIGH_SPEED);
        adc.setSamplingSpeed(ADC_SAMPLING_SPEED::HIGH_SPEED);
        
    }
    else if (sampleFrequency > 110000){
        
        //110kSPS - 220kSPS
        
        sample_resolution = 12;
        
        adc.setAveraging(num_avgs);
        adc.setResolution(sample_resolution);
    
        adc.setConversionSpeed(ADC_CONVERSION_SPEED::HIGH_SPEED);
        adc.setSamplingSpeed(ADC_SAMPLING_SPEED::MED_SPEED);
        
    }
    else if (sampleFrequency > 55000){
        
        //55kSPS - 110kSPS
        
        num_avgs = 2;
        sample_resolution = 12;
        
        adc.setAveraging(num_avgs);
        adc.setResolution(sample_resolution);
    
        adc.setConversionSpeed(ADC_CONVERSION_SPEED::HIGH_SPEED);
        adc.setSamplingSpeed(ADC_SAMPLING_SPEED::MED_SPEED);
        
    }
    else if (sampleFrequency > 40000){
        
        //40kSPS - 55kSPS
        
        num_avgs = 4;
        sample_resolution = 12;
        
        adc.setAveraging(num_avgs);
        adc.setResolution(sample_resolution);
    
        adc.setConversionSpeed(ADC_CONVERSION_SPEED::HIGH_SPEED);
        adc.setSamplingSpeed(ADC_SAMPLING_SPEED::MED_SPEED);
        
        
    }
    else if (sampleFrequency > 20000){
        
        //20kSPS - 40kSPS
        
        num_avgs = 4;
        sample_resolution = 12;
        
        adc.setAveraging(num_avgs);
        adc.setResolution(sample_resolution);
    
        adc.setConversionSpeed(ADC_CONVERSION_SPEED::MED_SPEED);
        adc.setSamplingSpeed(ADC_SAMPLING_SPEED::MED_SPEED);
        
    }
    else if (sampleFrequency > 10000){
        
        //10kSPS - 20kSPS
        
        num_avgs = 8;
        sample_resolution = 12;
        
        adc.setAveraging(num_avgs);
        adc.setResolution(sample_resolution);
    
        adc.setConversionSpeed(ADC_CONVERSION_SPEED::MED_SPEED);
        adc.setSamplingSpeed(ADC_SAMPLING_SPEED::MED_SPEED);
        
        
    }
    else if (sampleFrequency > 5000){
        
        //5kSPS - 10kSPS
        
        num_avgs = 16;
        sample_resolution = 12;
        
        adc.setAveraging(num_avgs);
        adc.setResolution(sample_resolution);
    
        adc.setConversionSpeed(ADC_CONVERSION_SPEED::MED_SPEED);
        adc.setSamplingSpeed(ADC_SAMPLING_SPEED::MED_SPEED);
        
        
    }
    else {
        
        //1kSPS - 5kSPS
        
        num_avgs = 32;
        sample_resolution = 12;
        
        adc.setAveraging(num_avgs);
        adc.setResolution(sample_resolution);
    
        adc.setConversionSpeed(ADC_CONVERSION_SPEED::MED_SPEED);
        adc.setSamplingSpeed(ADC_SAMPLING_SPEED::MED_SPEED);
        
    }
    
        //Standard setup stuff.
        adc.adc0->analogRead(adc_pin); 
        adc.enableInterrupts(ADC_0);
        NVIC_SET_PRIORITY(IRQ_ADC0,30);
        //ADC setup stuff, clears out garbage I assume, no real idea.
        adc.adc0->resetError();
        adc.adc0->stopPDB();
        
        //reset the ringbuffer to 0, not really necessary here, doesn't hurt I suppose.
        (buffer->b_end) = (buffer->b_start);
  
        
        adc.adc0->startPDB(pdb_trigger_frequency);
}

void Analogger::makeConnection(){

    Serial.begin(921600); //Speed doesn't matter, always at native USB speed.
 
    clearRxBuffer(1000);
    

    //Blink so you know something good has happened
    digitalWrite(LEDpin, HIGH);
    
    delay(1000);
    
    digitalWrite(LEDpin, LOW);

    //Until the host computer is ready, send messages saying we think configuration is done and we can move on to the next phase
    while(true){

        clearRxBuffer(1000);
        
        sendMessage(ANA_TYPE_ECHO, ANA_SUB_DELAY, MARCO);
        
        if(waitForMessage(3000, reply)){

            parseMessage(reply, replyType, replySub, replyValue);

            if(replyType == ANA_TYPE_ECHO){
                  
                  if(replyValue == POLO){
                      
                      break;
                    
                  }
            }
        }
      
    }
    
    //Tell teensy, we are connected
    isConnected = true;
    
    //Blink so you know something good has happened
    digitalWrite(LEDpin, HIGH);
    
    delay(1000);
    
    digitalWrite(LEDpin, LOW);
      
}

void Analogger::clearRxBuffer(uint32_t clearTime){
    
    uint32_t startTime = millis();
    
    while(true){

      if(Serial.available() > 0){
        
        Serial.read();
          
        startTime = millis();
        
      }

      delay(10);

      if ((startTime + clearTime) < millis()){

          break;
        
      }
      
    }
    
    
}



void Analogger::Config(bool SWEnabled, uint32_t Freq, uint32_t recordTime, uint32_t Triggers, uint32_t Delay){
    
    if( SWEnabled){
        
            sw_trigger_enable = 1;
            
    }
    
    else{
        
        sw_trigger_enable = 0;
        
    }
    
    pdb_trigger_frequency = Freq;
    
    seconds = recordTime;
    
    num_of_triggers = Triggers;
    
    time_delay = Delay;
    
    //Set the LEDpin to blink mode
    pinMode(LEDpin, OUTPUT);

    //If this is the configuration on startup do this, otherwise, do not try to remake a connection.
    if (!isConnected){
        
        //Set Transmit Buffers to all zeros.
        for (int ii = 0; ii < message_size; ii++){
            
           zeros[ii] = 0;
            
           message[ii] = 0;
            
           reply[ii] = 0;
        
        }

        //Make a connection to the host computer
        makeConnection();
    
    }
    
    bool settingUp = true;

    //Make sure teensy and computer communicate
    while(settingUp){

        //wait for a message
        if(waitForMessage(5000, reply)){

            clearRxBuffer(1000);
            
            //respond to message
            replyToMessage(reply);
            
        }

         //If the message told the teensy to begin, break out of this loop and move on with our lives.
         if (AnaloggerStatus == ADC_START){
             
             settingUp = false;
             
         }
            
    }

    //Set up the ADC to record at the sample frequency
    adc_setup(pdb_trigger_frequency);
    
    //Set the number of samples to be taken, if seconds is negative, it will just continue for 4.3 Billion samples.
    //I should add an infinite version, for tomorrow.
    if (seconds >= 0){
        
        numSamples = (pdb_trigger_frequency * seconds)/1000;
        
    }
    
    else{
        
        numSamples = 4294967295;
        
    }

    digitalWrite(LEDpin, LOW);

    pinMode(adc_pin, INPUT);
        
    pinMode(hw_trigger_pin, INPUT);
  
}

void Analogger::createMessage(uint8_t Type, uint8_t Sub, uint32_t Value){

    //If for some reason we try to create a data message with this function, don't.
    if (Type != ANA_TYPE_DATA){

        //First byte is the type of message, dictionary is in Analogger.h
        message[0] = Type;

        //Second byte is the parameter we are writing a message about, dictionary is in Analogger.h
        message[1] = Sub;

        //3rd - 6th bytes are the data values, I'm alway confused if this is big or little endian.
        message[2] = (Value >> 24) & 0xFF;
        
        message[3] = (Value >> 16) & 0xFF;
        
        message[4] = (Value >> 8) & 0xFF;
        
        message[5] = Value & 0xFF;

        //Calculate the checksum.  Since this is USB data, this really should not be necessary, but there is so much extra space in these messages, why not?
        int checkSum = 0;
        
        for (int ii = 0; ii < 6; ii++){
            
            checkSum += message[ii];
        
            
        }

        //7th and 8th bytes are the checksum we just calculated.
        message[6] = (checkSum >> 8) & 0XFF;
        
        message[7] = checkSum & 0XFF;

    }
  
}

void Analogger::createMessage(uint8_t Type, uint32_t SerialNum){

        //If for some reason you are trying to send other than data with this message, don't
        if (Type == ANA_TYPE_DATA){

            //Let people know it's data
            message[0] = Type;

            //The sub type is the serial number of the data message. I use this to make sure all the messages came through successfully.
            //If you are confident in your data pipe, you can send an extra three bytes of data here instead.  That's significant at high speeds
            //and works with both 12 bit and 8 bit readings.  The host computer code would need to be modified to accomodate this change as well.
            message[1] = (SerialNum >> 16) & 0xFF;
            
            message[2] = (SerialNum >> 8) & 0xFF;
            
            message[3] = SerialNum & 0xFF;
                
        }

        //Data actually gets written elsewhere, this is just for the header bytes
                
}


bool Analogger::waitForMessage(uint32_t waitTime, uint8_t reply[64]){

        uint8_t bytesRead = 0;

        //When did we start waiting?
        uint32_t startTime = millis();

        //How long have we been waiting?
        uint32_t elapsedTime = millis() - startTime;

        //If we should still be waiting, do this
        while (elapsedTime < waitTime){

            //If data has arrived, move it to the reply buffer
            if(Serial.available() > 0){

                //add the data to the next index in the buffer
                reply[bytesRead] = Serial.read();

                //count how many bytes we have received
                bytesRead++;
            
            }

            //If we counted 64 bytes, say we received a message
            if(bytesRead == 64){

                return(true);
              
            }

            //If we are less than 64 bytes, keep waiting for more
            else{

                //wait just a bit
                delay(5);

                //count how much time has passed.
                elapsedTime = millis() - startTime;
                  
            }
            
        }

     //If we couldn't get a whole message, just give up.
     return(false);
            
}


void Analogger::parseMessage(uint8_t tdxArray[64], uint8_t &Type, uint8_t &Sub, uint32_t &Value){
    
    //Check to see if this is a data message
    if (tdxArray[0] == ANA_TYPE_DATA){
        
        // Do nothing, don't need to receive data messages on the teensy side.
        Type = 0;
        
        Sub = 0;
        
        Value = 0;
        
        return;
        
    }

    //If it is not data, actually parse the message
    else{

        //read out the value bytes
        uint8_t val1 = tdxArray[2];
        
        uint8_t val2 = tdxArray[3];
        
        uint8_t val3 = tdxArray[4];
        
        uint8_t val4 = tdxArray[5];

        //Calculate the checksum
        int checkSum = (tdxArray[6] << 8) + tdxArray[7];

            //If the checksum matches, return the parsed message
            if ((tdxArray[0] + tdxArray[1] + val1 + val2 + val3 + val4) == checkSum){

                Type = tdxArray[0];
        
                Sub = tdxArray[1];
                
                Value = (val1 << 24) + (val2 << 16) + (val3 << 8) + val4;
            
                return;
            }

            //If the checksum did not match, return nothing
            else{

                Type = 0;
                
                Sub = 0;
                
                Value = 0;
                
                return;
                
            }       
        
    }
    
}


uint32_t Analogger::sendMessage(uint8_t Type, uint8_t SubType, uint32_t Value){
    
    //create a message to send   
    createMessage(Type, SubType, Value);

    //Send it
    Serial.write(message, 64);

    //As long as it is not Data (which it shouldn't be for this function)
    if (Type != ANA_TYPE_DATA){

       //Wait for a reply
       bool thisFlag = waitForMessage(1, reply);

        //If we got a reply
        if (thisFlag){

            //If the reply is an echo to our message, and we did not send an echo
            if ((reply[0] == ANA_TYPE_ECHO) & (Type != ANA_TYPE_ECHO)){

                //If we pushed data, just assume the computer handled it correctly.  Probably should check, but really if we got this far it is probably fine.
                if (Type == ANA_TYPE_PUSH){
                    
                    return (Value);
                    
                }

                //If instead we requested data
                else{

                    //Parse the reply
                    parseMessage(reply, replyType, replySub, replyValue);

                    //As long as a value was written, return it.
                    if (replyType > 0){
                        
                        return (replyValue);
                    
                    }
                    
                    //FOR ALL OTHER CASES NOT HANDLED ABOVE, RETURN 0 
                    else{
                        
                        return(0);
                    }
                    
                }
            }
            else{
                
                return (0);
            }
            
        }
        else{
            
            return (0);
        }
        
    }
    else{
        
        return (0);
    }    
    
}

void Analogger::sendMessage(uint8_t Type, uint32_t SerNum){

    //This is the function for sending data, just do it, needs to be fast.
    createMessage(Type, SerNum);
    
    Serial.write(message, 64);
    
}


void Analogger::replyToMessage(uint8_t message[64]){

    //Parse the message first, see what we should do.
    parseMessage(message, replyType, replySub, replyValue);

    //If it was a push, as in the computer is telling teensy what to change a value to, do that and reply with an echo
    //I did not use an unordered map, probably should have so this isn't a nightmare.
    if (replyType == ANA_TYPE_PUSH){
        
        if (replySub == ANA_SUB_FREQ){
            
            pdb_trigger_frequency = replyValue;
            
            sendMessage(ANA_TYPE_ECHO, replySub, replyValue);
            
        }
        else if (replySub == ANA_SUB_SEC){
                        
            seconds = replyValue;
            
            sendMessage(ANA_TYPE_ECHO, replySub, replyValue);
            
        }
        else if (replySub == ANA_SUB_NUM_TRIG){
                        
            num_of_triggers = replyValue;
            
            sendMessage(ANA_TYPE_ECHO, replySub, replyValue);
            
        }
        else if (replySub == ANA_SUB_DELAY){
                        
            time_delay = replyValue;
            
            sendMessage(ANA_TYPE_ECHO, replySub, replyValue);
            
        }
        else if (replySub == ANA_SUB_HWPIN){
                        
            //hw_trigger_pin = replyValue;
            
            sendMessage(ANA_TYPE_ECHO, replySub, replyValue);
            
        }
        else if (replySub == ANA_SUB_HWENAB){
                        
            hw_trigger_enable = replyValue;
            
            sendMessage(ANA_TYPE_ECHO, replySub, replyValue);
            
        }
        else if (replySub == ANA_SUB_ADCPIN){
                        
            //adc_pin = replyValue;
            
            sendMessage(ANA_TYPE_ECHO, replySub, replyValue);
            
        }
        else if (replySub == ANA_SUB_SWENAB){
                        
            sw_trigger_enable = replyValue;
            
            sendMessage(ANA_TYPE_ECHO, replySub, replyValue);
            
        }
        else if (replySub == ANA_SUB_CONFIG_ENAB){
                        
            config_enabled = replyValue;
            
            sendMessage(ANA_TYPE_ECHO, replySub, replyValue);
            
        }
        else if (replySub == ANA_SUB_ADCSTATUS){
                        
            AnaloggerStatus = replyValue;
            
            sendMessage(ANA_TYPE_ECHO, replySub, replyValue);
            
        }
        
    }

    //If instead there was a request for data made, send back the data
    else if(replyType == ANA_TYPE_REQ){
        
        if (replySub == ANA_SUB_FREQ){
            
            sendMessage(ANA_TYPE_ECHO, replySub, pdb_trigger_frequency);
            
        }
        else if (replySub == ANA_SUB_SEC){
            
            sendMessage(ANA_TYPE_ECHO, replySub, seconds);
            
        }
        else if (replySub == ANA_SUB_NUM_TRIG){
            
            sendMessage(ANA_TYPE_ECHO, replySub, num_of_triggers);
            
        }
        else if (replySub == ANA_SUB_DELAY){
            
            sendMessage(ANA_TYPE_ECHO, replySub, time_delay);
            
        }
        else if (replySub == ANA_SUB_HWPIN){
            
            sendMessage(ANA_TYPE_ECHO, replySub, hw_trigger_pin);
            
        }
        else if (replySub == ANA_SUB_HWENAB){
            
            sendMessage(ANA_TYPE_ECHO, replySub, hw_trigger_enable);
            
        }
        else if (replySub == ANA_SUB_ADCPIN){
            
            sendMessage(ANA_TYPE_ECHO, replySub, adc_pin);
            
        }
        else if (replySub == ANA_SUB_SWENAB){
            
            sendMessage(ANA_TYPE_ECHO, replySub, sw_trigger_enable);
            
        }
        else if (replySub == ANA_SUB_CONFIG_ENAB){
            
            sendMessage(ANA_TYPE_ECHO, replySub, config_enabled);
            
        }
        else if (replySub == ANA_SUB_ADCSTATUS){
            
            sendMessage(ANA_TYPE_ECHO, replySub, AnaloggerStatus);
            
        }
        
    }
    
    //If the message was an echo, no reply needed.
    else if (replyType == ANA_TYPE_ECHO){
        
        
    }

    //If it was data, not sure how we got here, but no reply needed.
    else if (replyType == ANA_TYPE_DATA){
        
    }

}

void Analogger::analoggerTrigger(){
      
        //Record when trigger was activated.
        time_trigger_start = millis();

        
        
    while((millis() - time_trigger_start) < (time_delay)){
        //Just block until the delay is up.  I would like a non-blocking delay, but that is a problem for another day.
    }
        
        //We don't want the data before the trigger was activated, but we don't want to waste time initializing the pdb after the trigger, so here we are hijacking the ring buffer code.
        (buffer->b_start) = (buffer->b_end);
        randomCounter = true;

        //Initialize the overrun variables
        overrun = false; //may have 'overrun' before we even started recording real data, set to 0 to clear
        
        error_count = 0;
        
        start_read = false;
        

        while(readCounter < numSamples){   //Keep reading if we have not hit our number of samples.
            
            if (!(buffer->isEmpty())){                
                
              ADCRecord(buffer, message, tx_buffer_index, sample_resolution);
              
              readCounter++;

            }

              //  >>>>>>>  NOTE <<<< 
              //    This will always return the number of asked for samples regardless of whether or not the time for them is well past
              //    Since the ADC is dutifully running the entire time this loop is, if the ADC buffer overruns, this loop doesn't care
              //    This loop will just pull from the ring buffer, whatever is there and send it.  I need to decide whether or not I
              //    want this type of behaivor, or if I would like to have holes in the data (maybe?), or something else....  
              

              if(overrun){
              
                  error_count = readCounter - 1;  //This will be the last trusted value of the readings.  Anything after this may have been an overrun value. 

                  overrun = false;
                          
              }
              
              if (tx_buffer_index == message_size){
                  
                  sendMessage(ANA_TYPE_DATA, Serial_Number);

                  Serial_Number++;

                  tx_buffer_index = 4;
              
              }
        }
        
        notTriggered = true;
    
    if (tx_buffer_index > 4){
        
        for (int ii = tx_buffer_index + 1; ii < message_size; ii ++){
            
            message[ii] = 0;   
        
        }
        
        sendMessage(ANA_TYPE_DATA, Serial_Number);
        
        Serial_Number ++;
        
        tx_buffer_index = 4;
        
    }
    
    readCounter = 0;
    
        
    sendMessage(ANA_TYPE_PUSH, ANA_SUB_ADCSTATUS, ADC_END);

    delay(10);

    sendMessage(ANA_TYPE_PUSH, ANA_SUB_ADCSTATUS, error_count);
    
    
}

void Analogger::hardwareTrigger(){

    //If we are still expecting more triggers to fire, check if it has
    if(num_of_triggers > 0){

            //If the trigger pin is high, go for it.
            if(digitalReadFast(hw_trigger_pin)){ 

                  //Trigger the reading loops
                  analoggerTrigger();
                  
                  // decement the triggers now that one has fired
                  num_of_triggers--;
              
            }
    }
    
}


void Analogger::softwareTrigger(){

    //If we are still waiting for more triggers to fire, check to see if it has
    if(num_of_triggers > 0){      
              
            if(waitForMessage(2000, reply)){                
                
                parseMessage(reply, replyType, replySub, replyValue);

                if(replyType == ANA_TYPE_PUSH){
                    
                    if(replyValue == SW_TRIGGER){
                        
                        replyToMessage(reply);
                        
                        analoggerTrigger();
                        
                        // decrement the number of triggers now that one has fired
                        num_of_triggers--;
                        
                    }
                }
            }
            
            clearRxBuffer(10);
    }
    
}
